@@ -31,28 +31,25 @@
 int
 p4d_same(const POINT4D *p1, const POINT4D *p2)
 {
-	if( FP_EQUALS(p1->x,p2->x) && FP_EQUALS(p1->y,p2->y) && FP_EQUALS(p1->z,p2->z) && FP_EQUALS(p1->m,p2->m) )
-		return LW_TRUE;
-	else
-		return LW_FALSE;
+	return p1->x == p2->x
+	    && p1->y == p2->y
+	    && p1->z == p2->z
+	    && p1->m == p2->m;
 }
 
 int
 p3d_same(const POINT3D *p1, const POINT3D *p2)
 {
-	if( FP_EQUALS(p1->x,p2->x) && FP_EQUALS(p1->y,p2->y) && FP_EQUALS(p1->z,p2->z) )
-		return LW_TRUE;
-	else
-		return LW_FALSE;
+	return p1->x == p2->x
+	    && p1->y == p2->y
+	    && p1->z == p2->z;
 }
 
 int
 p2d_same(const POINT2D *p1, const POINT2D *p2)
 {
-	if( FP_EQUALS(p1->x,p2->x) && FP_EQUALS(p1->y,p2->y) )
-		return LW_TRUE;
-	else
-		return LW_FALSE;
+	return p1->x == p2->x
+	    && p1->y == p2->y;
 }
 
 /**
@@ -85,7 +82,9 @@ lw_seg_length(const POINT2D *A1, const POINT2D *A2)
 int
 lw_pt_in_arc(const POINT2D *P, const POINT2D *A1, const POINT2D *A2, const POINT2D *A3)
 {
-	return lw_segment_side(A1, A3, A2) == lw_segment_side(A1, A3, P);
+	int pt_side = lw_segment_side(A1, A3, P);
+	int arc_side = lw_segment_side(A1, A3, A2);
+	return pt_side == 0 || pt_side == arc_side;
 }
 
 /**
@@ -99,6 +98,14 @@ lw_pt_in_seg(const POINT2D *P, const POINT2D *A1, const POINT2D *A2)
 	       ((A1->y <= P->y && P->y < A2->y) || (A1->y >= P->y && P->y > A2->y));
 }
 
+int
+lw_pt_on_segment(const POINT2D* A1, const POINT2D* A2, const POINT2D* P)
+{
+	int side = lw_segment_side(A1, A2, P);
+	if (side != 0) return LW_FALSE;
+	return lw_pt_in_seg(P, A1, A2);
+}
+
 /**
 * Returns true if arc A is actually a point (all vertices are the same) .
 */
