@@ -25,6 +25,16 @@
 #include "lwgeom_sfcgal.h"
 #include <stdio.h>
 
+#if POSTGIS_SFCGAL_VERSION >= 20100
+#define sfcgal_triangulated_surface_num_triangles(g) sfcgal_triangulated_surface_num_patches((g))
+#define sfcgal_triangulated_surface_triangle_n(g, i) sfcgal_triangulated_surface_patch_n((g), (i))
+#define sfcgal_triangulated_surface_add_triangle(g, p) sfcgal_triangulated_surface_add_patch((g), (p))
+#define sfcgal_polyhedral_surface_num_polygons(g) sfcgal_polyhedral_surface_num_patches((g))
+#define sfcgal_polyhedral_surface_polygon_n(g, i) sfcgal_polyhedral_surface_patch_n((g), (i))
+#define sfcgal_geometry_collection_num_geometries(g) sfcgal_geometry_num_geometries((g))
+#define sfcgal_polyhedral_surface_add_polygon(g, p) sfcgal_polyhedral_surface_add_patch((g), (p))
+#endif
+
 static int SFCGAL_type_to_lwgeom_type(sfcgal_geometry_type_t type);
 static POINTARRAY *ptarray_from_SFCGAL(const sfcgal_geometry_t *geom, int force3D);
 static sfcgal_geometry_t *ptarray_to_SFCGAL(const POINTARRAY *pa, int type);
@@ -45,8 +55,10 @@ lwgeom_sfcgal_full_version()
 #if POSTGIS_SFCGAL_VERSION >= 10400
 	const char *version = sfcgal_full_version();
 #else
-	char *version = (char*)malloc(MAX_LENGTH_SFCGAL_FULL_VERSION);
-	snprintf(version, MAX_LENGTH_SFCGAL_FULL_VERSION, "SFCGAL=\"%s\" CGAL=\"Unknown\" Boost=\"Unknown\"", sfcgal_version());
+	char *version = (char*)lwalloc(MAX_LENGTH_SFCGAL_FULL_VERSION);
+	snprintf(version, MAX_LENGTH_SFCGAL_FULL_VERSION,
+		"SFCGAL=\"%s\" CGAL=\"Unknown\" Boost=\"Unknown\"",
+		sfcgal_version());
 #endif
 	return version;
 }
