@@ -25,7 +25,6 @@
 #include "utils/syscache.h"
 
 #include "liblwgeom.h"
-#include "pgsql_compat.h"
 
 #if POSTGIS_PGSQL_VERSION > 150
 #include "varatt.h"
@@ -89,13 +88,23 @@ void pg_install_lwgeom_handlers(void);
 
 /* Argument handling macros */
 #define PG_GETARG_GSERIALIZED_P(varno) ((GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(varno)))
+
 #define PG_GETARG_GSERIALIZED_P_COPY(varno) ((GSERIALIZED *)PG_DETOAST_DATUM_COPY(PG_GETARG_DATUM(varno)))
+
 #define PG_GSERIALIZED_DATUM_NEEDS_DETOAST(datum) \
 	(VARATT_IS_EXTENDED((datum)) || VARATT_IS_EXTERNAL((datum)) || VARATT_IS_COMPRESSED((datum)))
+
+#if POSTGIS_PGSQL_VERSION >= 190
+#define PG_GETARG_GSERIALIZED_HEADER(varno) \
+	PG_GSERIALIZED_DATUM_NEEDS_DETOAST(DatumGetPointer(PG_GETARG_DATUM(varno))) \
+	? ((GSERIALIZED *)PG_DETOAST_DATUM_SLICE(PG_GETARG_DATUM(varno), 0, gserialized_max_header_size())) \
+	: ((GSERIALIZED *)(PG_GETARG_POINTER(varno)))
+#else
 #define PG_GETARG_GSERIALIZED_HEADER(varno) \
 	PG_GSERIALIZED_DATUM_NEEDS_DETOAST(PG_GETARG_DATUM(varno)) \
 	? ((GSERIALIZED *)PG_DETOAST_DATUM_SLICE(PG_GETARG_DATUM(varno), 0, gserialized_max_header_size())) \
 	: ((GSERIALIZED *)(PG_GETARG_DATUM(varno)))
+#endif
 
 /* Debugging macros */
 #if POSTGIS_DEBUG_LEVEL > 0
