@@ -86,9 +86,11 @@ main (int argc, char **argv)
 	int ret, i;
 
 #ifdef ENABLE_NLS
+#ifdef PGSQL_LOCALEDIR
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, PGSQL_LOCALEDIR);
 	textdomain (PACKAGE);
+#endif
 #endif
 
 	/* If no options are specified, display usage */
@@ -103,7 +105,7 @@ main (int argc, char **argv)
 	set_loader_config_defaults(config);
 
 	/* Keep the flag list alphabetic so it's easy to see what's left. */
-	while ((c = pgis_getopt(argc, argv, "-acdeg:ikm:nps:t:wDGIN:ST:W:X:Z")) != EOF)
+	while ((c = pgis_getopt(argc, argv, "-?acdeg:ikm:nps:t:wDGIN:ST:W:X:Z")) != EOF)
 	{
 		// can not do this inside the switch case
 		if ('-' == c)
