@@ -25,7 +25,7 @@
 #include <math.h>
 #include "geobuf.h"
 
-#if defined HAVE_LIBPROTOBUF && defined HAVE_GEOBUF
+#if defined HAVE_LIBPROTOBUF
 
 #define FEATURES_CAPACITY_INITIAL 50
 #define MAX_PRECISION 1e6
@@ -57,13 +57,8 @@ static void encode_keys(struct geobuf_agg_context *ctx)
 	uint32_t i, k = 0;
 	bool geom_found = false;
 	for (i = 0; i < natts; i++) {
-#if POSTGIS_PGSQL_VERSION < 110
-		Oid typoid = getBaseType(tupdesc->attrs[i]->atttypid);
-		char *tkey = tupdesc->attrs[i]->attname.data;
-#else
-		Oid typoid = getBaseType(tupdesc->attrs[i].atttypid);
-		char *tkey = tupdesc->attrs[i].attname.data;
-#endif
+		Oid typoid = getBaseType(TupleDescAttr(tupdesc, i)->atttypid);
+		char *tkey = TupleDescAttr(tupdesc, i)->attname.data;
 		char *key = pstrdup(tkey);
 		if (ctx->geom_name == NULL) {
 			if (!geom_found && typoid == postgis_oid(GEOMETRYOID)) {
@@ -94,7 +89,7 @@ static void set_int_value(Data__Value *value, int64 intval) {
 		value->pos_int_value = (uint64_t) intval;
 	} else {
 		value->value_type_case = DATA__VALUE__VALUE_TYPE_NEG_INT_VALUE;
-		value->neg_int_value = (uint64_t) labs(intval);
+		value->neg_int_value = (uint64_t)llabs(intval);
 	}
 }
 
@@ -127,11 +122,9 @@ static void encode_properties(struct geobuf_agg_context *ctx,
 		datum = GetAttributeByNum(ctx->row, i + 1, &isnull);
 		if (isnull)
 			continue;
-#if POSTGIS_PGSQL_VERSION < 110
-		typoid = getBaseType(tupdesc->attrs[i]->atttypid);
-#else
-		typoid = getBaseType(tupdesc->attrs[i].atttypid);
-#endif
+
+		typoid = getBaseType(TupleDescAttr(tupdesc, i)->atttypid);
+
 		if (strcmp(type, "int2") == 0) {
 			set_int_value(value, DatumGetInt16(datum));
 		} else if (strcmp(type, "int4") == 0) {
@@ -259,6 +252,23 @@ static Data__Geometry *encode_line(struct geobuf_agg_context *ctx,
 	return geometry;
 }
 
+static Data__Geometry *
+encode_triangle(struct geobuf_agg_context *ctx, LWTRIANGLE *lwtri)
+{
+	Data__Geometry *geometry = galloc(DATA__GEOMETRY__TYPE__POLYGON);
+	POINTARRAY *pa = lwtri->points;
+	uint32_t len;
+
+	if (pa->npoints == 0)
+		return geometry;
+
+	len = pa->npoints - 1;
+	geometry->n_coords = len * ctx->dimensions;
+	geometry->coords = encode_coords(ctx, pa, NULL, len, 0);
+
+	return geometry;
+}
+
 static Data__Geometry *encode_mline(struct geobuf_agg_context *ctx,
 	LWMLINE *lwmline)
 {
@@ -421,6 +431,8 @@ static Data__Geometry *encode_geometry(struct geobuf_agg_context *ctx,
 		return encode_point(ctx, (LWPOINT*)lwgeom);
 	case LINETYPE:
 		return encode_line(ctx, (LWLINE*)lwgeom);
+	case TRIANGLETYPE:
+		return encode_triangle(ctx, (LWTRIANGLE *)lwgeom);
 	case POLYGONTYPE:
 		return encode_poly(ctx, (LWPOLY*)lwgeom);
 	case MULTIPOINTTYPE:
@@ -430,6 +442,7 @@ static Data__Geometry *encode_geometry(struct geobuf_agg_context *ctx,
 	case MULTIPOLYGONTYPE:
 		return encode_mpoly(ctx, (LWMPOLY*)lwgeom);
 	case COLLECTIONTYPE:
+	case TINTYPE:
 		return encode_collection(ctx, (LWCOLLECTION*)lwgeom);
 	default:
 		elog(ERROR, "encode_geometry: '%s' geometry type not supported",
@@ -471,6 +484,7 @@ static void analyze_geometry(struct geobuf_agg_context *ctx, LWGEOM *lwgeom)
 	{
 	case POINTTYPE:
 	case LINETYPE:
+	case TRIANGLETYPE:
 		lwline = (LWLINE*) lwgeom;
 		analyze_pa(ctx, lwline->points);
 		break;
@@ -483,6 +497,7 @@ static void analyze_geometry(struct geobuf_agg_context *ctx, LWGEOM *lwgeom)
 	case MULTILINETYPE:
 	case MULTIPOLYGONTYPE:
 	case COLLECTIONTYPE:
+	case TINTYPE:
 		lwcollection = (LWCOLLECTION*) lwgeom;
 		for (i = 0; i < lwcollection->ngeoms; i++)
 			analyze_geometry(ctx, lwcollection->geoms[i]);
@@ -496,11 +511,12 @@ static void analyze_geometry(struct geobuf_agg_context *ctx, LWGEOM *lwgeom)
 static void analyze_geometry_flags(struct geobuf_agg_context *ctx,
 	LWGEOM *lwgeom)
 {
-	if (!ctx->has_dimensions) {
-		if (FLAGS_GET_Z(lwgeom->flags) || FLAGS_GET_M(lwgeom->flags))
-			ctx->dimensions = 3;
-		else if (FLAGS_GET_ZM(lwgeom->flags))
+	if (!ctx->has_dimensions)
+	{
+		if (lwgeom_has_z(lwgeom) && lwgeom_has_m(lwgeom))
 			ctx->dimensions = 4;
+		else if (lwgeom_has_z(lwgeom) || lwgeom_has_m(lwgeom))
+			ctx->dimensions = 3;
 		else
 			ctx->dimensions = 2;
 		ctx->has_dimensions = 1;
