@@ -1509,7 +1509,11 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 		notnull_cnt++;
 
 		/* Give backend a chance of interrupting us */
+#if POSTGIS_PGSQL_VERSION >= 180
+		vacuum_delay_point(true);
+#else
 		vacuum_delay_point();
+#endif
 	}
 
 	/*
@@ -1727,8 +1731,11 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 		if ( ! nd_box ) continue; /* Skip Null'ed out hard deviants */
 
 		/* Give backend a chance of interrupting us */
+#if POSTGIS_PGSQL_VERSION >= 180
+		vacuum_delay_point(true);
+#else
 		vacuum_delay_point();
-
+#endif
 		/* Find the cells that overlap with this box and put them into the ND_IBOX */
 		nd_box_overlap(nd_stats, nd_box, &nd_ibox);
 		memset(at, 0, sizeof(int)*ND_DIMS);
@@ -2267,7 +2274,7 @@ gserialized_sel_internal(PlannerInfo *root, List *args, int varRelid, int mode)
 	nd_stats = pg_nd_stats_from_tuple(vardata.statsTuple, mode);
 	ReleaseVariableStats(vardata);
 	selectivity = estimate_selectivity(&search_box, nd_stats, mode);
-	pfree(nd_stats);
+	if (nd_stats) pfree(nd_stats);
 	return selectivity;
 }
 
