@@ -27,7 +27,6 @@
 #include <float.h>
 #include <string.h>
 #include <stdio.h>
-#include <errno.h>
 #include <assert.h>
 
 #include "postgres.h"
@@ -145,7 +144,7 @@ Datum LWGEOM_dump(PG_FUNCTION_ARGS)
 	if ( ! lwgeom_is_collection(state->root) )
 	{
 		values[0] = "{}";
-		values[1] = lwgeom_to_hexwkb(state->root, WKB_EXTENDED, 0);
+		values[1] = lwgeom_to_hexwkb_buffer(state->root, WKB_EXTENDED);
 		tuple = BuildTupleFromCStrings(funcctx->attinmeta, values);
 		result = HeapTupleGetDatum(tuple);
 
@@ -183,6 +182,9 @@ Datum LWGEOM_dump(PG_FUNCTION_ARGS)
 			 * stack
 			 */
 
+			if (state->stacklen > MAXDEPTH)
+				elog(ERROR, "Unable to dump overly nested collection.");
+
 			oldcontext = MemoryContextSwitchTo(newcontext);
 
 			node = lwalloc(sizeof(GEOMDUMPNODE));
@@ -202,7 +204,7 @@ Datum LWGEOM_dump(PG_FUNCTION_ARGS)
 	lwgeom->srid = state->root->srid;
 
 	values[0] = address;
-	values[1] = lwgeom_to_hexwkb(lwgeom, WKB_EXTENDED, 0);
+	values[1] = lwgeom_to_hexwkb_buffer(lwgeom, WKB_EXTENDED);
 	tuple = BuildTupleFromCStrings(funcctx->attinmeta, values);
 	result = TupleGetDatum(funcctx->slot, tuple);
 	node->idx++;
@@ -302,7 +304,7 @@ Datum LWGEOM_dump_rings(PG_FUNCTION_ARGS)
 		sprintf(address, "{%d}", state->ringnum);
 
 		values[0] = address;
-		values[1] = lwgeom_to_hexwkb(ringgeom, WKB_EXTENDED, 0);
+		values[1] = lwgeom_to_hexwkb_buffer(ringgeom, WKB_EXTENDED);
 
 		MemoryContextSwitchTo(oldcontext);
 
@@ -346,7 +348,9 @@ Datum ST_Subdivide(PG_FUNCTION_ARGS)
 		GSERIALIZED *gser;
 		LWGEOM *geom;
 		LWCOLLECTION *col;
-		int maxvertices = 256;
+		/* default to maxvertices < page size */
+		int maxvertices = 128;
+		double gridSize = -1;
 
 		/* create a function context for cross-call persistence */
 		funcctx = SRF_FIRSTCALL_INIT();
@@ -368,10 +372,16 @@ Datum ST_Subdivide(PG_FUNCTION_ARGS)
 		if ( PG_NARGS() > 1 && ! PG_ARGISNULL(1) )
 			maxvertices = PG_GETARG_INT32(1);
 
+		/*
+		* Get the gridSize value
+		*/
+		if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+			gridSize = PG_GETARG_FLOAT8(2);
+
 		/*
 		* Compute the subdivision of the geometry
 		*/
-		col = lwgeom_subdivide(geom, maxvertices);
+		col = lwgeom_subdivide_prec(geom, maxvertices, gridSize);
 
 		if ( ! col )
 			SRF_RETURN_DONE(funcctx);
