@@ -1615,8 +1615,9 @@ Datum ST_ClipByBox2d(PG_FUNCTION_ARGS)
 	bbox2 = (GBOX *)PG_GETARG_POINTER(box2d_idx);
 	bbox2->flags = 0;
 
-	/* if bbox1 is covered by bbox2, return lwgeom1 */
-	if (gbox_contains_2d(bbox2, &bbox1))
+	/* If bbox1 is strictly contained by bbox2, return input geometry */
+	if (bbox2->xmin < bbox1.xmin && bbox2->xmax > bbox1.xmax &&
+	    bbox2->ymin < bbox1.ymin && bbox2->ymax > bbox1.ymax)
 	{
 		PG_RETURN_DATUM(PG_GETARG_DATUM(geom_idx));
 	}
@@ -2612,6 +2613,7 @@ Datum relate_pattern(PG_FUNCTION_ARGS)
 	GSERIALIZED *geom1;
 	GSERIALIZED *geom2;
 	char *patt;
+	text *ptxt;
 	char result;
 	GEOSGeometry *g1, *g2;
 	size_t i;
@@ -2634,7 +2636,10 @@ Datum relate_pattern(PG_FUNCTION_ARGS)
 		HANDLE_GEOS_ERROR("Second argument geometry could not be converted to GEOS");
 	}
 
-	patt =  DatumGetCString(DirectFunctionCall1(textout, PG_GETARG_DATUM(2)));
+	/* Ensure DE9IM pattern is no more than 9 chars */
+	ptxt = DatumGetTextP(DirectFunctionCall2(text_left,
+	               PG_GETARG_DATUM(2), Int32GetDatum(9)));
+	patt = text_to_cstring(ptxt);
 
 	/*
 	** Need to make sure 't' and 'f' are upper-case before handing to GEOS
