@@ -258,3 +258,232 @@ IF _postgis_scripts_pgsql_version()::integer >= 96 THEN
 END IF;
 END;
 $$;
+
+-- #5564 fix up the brin op classes for upgrades
+DO language plpgsql
+$$
+BEGIN
+    -- Check if the function is already associated with any operator class
+    IF NOT EXISTS (
+        SELECT 1 FROM pg_catalog.pg_amproc
+        WHERE amproc::text = 'geom2d_brin_inclusion_merge' AND
+              amprocfamily IN (
+                  SELECT oid FROM pg_catalog.pg_opfamily
+                  WHERE opfname = 'brin_geometry_inclusion_ops_2d'
+              )
+    ) THEN
+        BEGIN
+            -- Create a temporary operator class for 'brin_geometry_inclusion_ops_2d_temp'
+            CREATE OPERATOR CLASS brin_geometry_inclusion_ops_2d_temp
+                FOR TYPE geometry USING brin AS
+                    FUNCTION 11 geom2d_brin_inclusion_merge(internal, internal);
+
+            -- find current and new operator family OIDs
+            WITH a AS (
+                SELECT n.oid AS oid_nfamily, o.oid AS oid_ofamily
+                FROM pg_catalog.pg_opfamily AS n
+                CROSS JOIN (
+                    SELECT oid
+                    FROM pg_catalog.pg_opfamily
+                    WHERE opfname = 'brin_geometry_inclusion_ops_2d_temp'
+                ) AS o
+                WHERE n.opfname = 'brin_geometry_inclusion_ops_2d'
+            ),
+            -- Update the amprocfamily in pg_amproc
+            amupdate AS (
+                UPDATE pg_catalog.pg_amproc AS amp
+                SET amprocfamily = a.oid_nfamily
+                FROM a
+                WHERE amp.amprocfamily = a.oid_ofamily AND
+                      amp.amproc::text = 'geom2d_brin_inclusion_merge'
+                RETURNING amp.*
+            )
+            -- Update dependencies in pg_depend to the existing operator class
+            UPDATE pg_depend AS d
+            SET refobjid = (
+                SELECT n.oid
+                FROM pg_catalog.pg_opclass AS n
+                WHERE opcname = 'brin_geometry_inclusion_ops_2d'
+            )
+            FROM amupdate
+            WHERE d.objid = amupdate.oid AND
+                  refobjid = (
+                      SELECT o.oid
+                      FROM pg_catalog.pg_opclass AS o
+                      WHERE opcname = 'brin_geometry_inclusion_ops_2d_temp'
+                  );
+
+            -- Drop the temporary operator family after use
+            DROP OPERATOR FAMILY brin_geometry_inclusion_ops_2d_temp USING brin;
+        EXCEPTION WHEN OTHERS THEN
+            RAISE EXCEPTION
+                'Could not add geom2d_brin_inclusion_merge to brin_geometry_inclusion_ops_2d class: %',
+                SQLERRM;
+        END;
+    END IF;
+
+    -- Check if the function is already associated with any operator class
+    IF NOT EXISTS (
+        SELECT 1
+        FROM pg_catalog.pg_amproc
+        WHERE amproc::text = 'geom3d_brin_inclusion_merge'
+    ) THEN
+        BEGIN
+            -- Create a temporary operator class with the new function
+            CREATE OPERATOR CLASS brin_geometry_inclusion_ops_3d_temp
+            FOR TYPE geometry USING brin AS
+                FUNCTION 11 geom3d_brin_inclusion_merge(internal, internal);
+
+            -- Change the pg_amproc association to the existing opclass
+             WITH a AS (
+                SELECT n.oid AS oid_nfamily, o.oid AS oid_ofamily
+                FROM pg_catalog.pg_opfamily AS n
+                CROSS JOIN (
+                    SELECT oid FROM pg_catalog.pg_opfamily
+                    WHERE opfname = 'brin_geometry_inclusion_ops_3d_temp'
+                ) AS o
+                WHERE n.opfname = 'brin_geometry_inclusion_ops_3d'
+            ), amupdate AS (
+                UPDATE pg_catalog.pg_amproc AS amp
+                SET amprocfamily = a.oid_nfamily
+                FROM a
+                WHERE
+                    amp.amprocfamily = a.oid_ofamily
+                    AND amp.amproc::text = 'geom3d_brin_inclusion_merge'
+                RETURNING amp.*
+            )
+            -- Change the opclass dependency to the existing one
+            UPDATE pg_depend AS d
+            SET refobjid = (
+                SELECT n.oid
+                FROM pg_catalog.pg_opclass AS n
+                WHERE opcname = 'brin_geometry_inclusion_ops_3d'
+            )
+            FROM amupdate
+            WHERE d.objid = amupdate.oid AND
+                  refobjid = (
+                    SELECT o.oid
+                    FROM pg_catalog.pg_opclass AS o
+                    WHERE opcname = 'brin_geometry_inclusion_ops_3d_temp'
+                );
+
+            -- Dropping the autogenerated temporary family which cascades to temporary class
+            DROP OPERATOR FAMILY brin_geometry_inclusion_ops_3d_temp USING brin;
+        EXCEPTION WHEN OTHERS THEN
+            RAISE EXCEPTION 'Could not add geom3d_brin_inclusion_merge to brin_geometry_inclusion_ops_3d class: %', SQLERRM;
+        END;
+    END IF;
+
+    -- Check if the function is already associated with any operator class
+    IF NOT EXISTS (
+        SELECT 1
+        FROM pg_catalog.pg_amproc
+        WHERE amproc::text = 'geom4d_brin_inclusion_merge'
+    ) THEN
+        BEGIN
+            -- Create a temporary operator class with the new function
+            CREATE OPERATOR CLASS brin_geometry_inclusion_ops_4d_temp
+            FOR TYPE geometry USING brin AS
+                FUNCTION 11 geom4d_brin_inclusion_merge(internal, internal);
+
+            -- Change the pg_amproc association to the existing opclass
+             WITH a AS (
+                SELECT n.oid AS oid_nfamily, o.oid AS oid_ofamily
+                FROM pg_catalog.pg_opfamily AS n
+                CROSS JOIN (
+                    SELECT oid FROM pg_catalog.pg_opfamily
+                    WHERE opfname = 'brin_geometry_inclusion_ops_4d_temp'
+                ) AS o
+                WHERE n.opfname = 'brin_geometry_inclusion_ops_4d'
+            ) , amupdate AS (
+                UPDATE pg_catalog.pg_amproc AS amp
+                SET amprocfamily = a.oid_nfamily
+                FROM a
+                WHERE
+                    amp.amprocfamily = a.oid_ofamily
+                    AND amp.amproc::text = 'geom4d_brin_inclusion_merge'
+                RETURNING amp.*
+            )
+            -- Change the opclass dependency to the existing one
+            UPDATE pg_depend AS d
+            SET refobjid = (
+                SELECT n.oid
+                FROM pg_catalog.pg_opclass AS n
+                WHERE opcname = 'brin_geometry_inclusion_ops_4d'
+            )
+            FROM amupdate
+            WHERE d.objid = amupdate.oid AND
+                  refobjid = (
+                    SELECT o.oid
+                    FROM pg_catalog.pg_opclass AS o
+                    WHERE opcname = 'brin_geometry_inclusion_ops_4d_temp'
+                );
+
+            -- Dropping the autogenerated temporary family also drops the temporary class
+            DROP OPERATOR FAMILY brin_geometry_inclusion_ops_4d_temp USING brin;
+        EXCEPTION WHEN OTHERS THEN
+            RAISE EXCEPTION
+                'Could not add geom4d_brin_inclusion_merge to brin_geometry_inclusion_ops_4d class: %',
+                SQLERRM;
+        END;
+    END IF;
+
+-- geography brin
+    -- Check if the function 'geog_brin_inclusion_merge' already exists
+    IF NOT EXISTS (
+        SELECT 1
+        FROM pg_catalog.pg_amproc
+        WHERE amproc::text = 'geog_brin_inclusion_merge'
+    ) THEN
+        BEGIN
+            -- Create a temporary operator class for 'brin_geography_inclusion_ops_temp'
+            CREATE OPERATOR CLASS brin_geography_inclusion_ops_temp
+                FOR TYPE geography USING brin AS
+                    FUNCTION 11 geog_brin_inclusion_merge(internal, internal);
+
+            -- find current and new operator family OIDs
+            WITH a AS (
+                SELECT n.oid AS oid_nfamily, o.oid AS oid_ofamily
+                FROM pg_catalog.pg_opfamily AS n
+                CROSS JOIN (
+                    SELECT oid
+                    FROM pg_catalog.pg_opfamily
+                    WHERE opfname = 'brin_geography_inclusion_ops_temp'
+                ) AS o
+                WHERE n.opfname = 'brin_geography_inclusion_ops'
+            ),
+            -- Update the amprocfamily in pg_amproc
+            amupdate AS (
+                UPDATE pg_catalog.pg_amproc AS amp
+                SET amprocfamily = a.oid_nfamily
+                FROM a
+                WHERE amp.amprocfamily = a.oid_ofamily AND
+                      amp.amproc::text = 'geog_brin_inclusion_merge'
+                RETURNING amp.*
+            )
+            -- Update dependencies in pg_depend to the existing operator class
+            UPDATE pg_depend AS d
+            SET refobjid = (
+                SELECT n.oid
+                FROM pg_catalog.pg_opclass AS n
+                WHERE opcname = 'brin_geography_inclusion_ops'
+            )
+            FROM amupdate
+            WHERE d.objid = amupdate.oid AND
+                  refobjid = (
+                      SELECT o.oid
+                      FROM pg_catalog.pg_opclass AS o
+                      WHERE opcname = 'brin_geography_inclusion_ops_temp'
+                  );
+
+            -- Drop the temporary operator family after use
+            DROP OPERATOR FAMILY brin_geography_inclusion_ops_temp USING brin;
+        EXCEPTION WHEN OTHERS THEN
+            RAISE EXCEPTION
+                'Could not add geog_brin_inclusion_merge to brin_geography_inclusion_ops class: %',
+                SQLERRM;
+        END;
+    END IF;
+END;
+
+$$;
