@@ -129,19 +129,27 @@
  *   datum is copied for use.
  *****************************************************************************/
 
-#include <postgres.h> /* for palloc */
-#include <fmgr.h> /* for PG_MODULE_MAGIC */
+/* PostgreSQL */
+#include "postgres.h" /* for palloc */
+#include "fmgr.h" /* for PG_MODULE_MAGIC */
+#include "libpq/pqsignal.h"
 #include "utils/guc.h"
+#include "utils/builtins.h"
 #include "utils/memutils.h"
+#include "utils/elog.h"
 
-#include "../../postgis_config.h"
+/* PostGIS */
+#include "postgis_config.h"
+#include "liblwgeom.h"
 #include "lwgeom_pg.h"
-
-#include "rtpostgis.h"
-#include "rtpg_internal.h"
 #include "stringlist.h"
 #include "optionlist.h"
 
+/* PostGIS Raster */
+#include "rtpostgis.h"
+#include "rtpg_internal.h"
+
+
 #ifndef __GNUC__
 # define __attribute__ (x)
 #endif
@@ -259,7 +267,7 @@ rt_pg_options(const char* varname)
 /*  GDAL allowed config options for VSI filesystems */
 /* ---------------------------------------------------------------- */
 
-stringlist_t *vsi_option_stringlist = NULL;
+static stringlist_t *vsi_option_stringlist = NULL;
 
 
 #if POSTGIS_GDAL_VERSION < 23
@@ -452,7 +460,6 @@ rt_pg_vsi_check_options(char **newval, void **extra, GucSource source)
 
 static char *gdal_datapath = NULL;
 static char *gdal_vsi_options = NULL;
-
 static char *gdal_enabled_drivers = NULL;
 static bool enable_outdb_rasters = false;
 
@@ -805,9 +812,9 @@ _PG_init(void) {
 void
 _PG_fini(void) {
 
-	MemoryContext old_context;
+	MemoryContext old_context = MemoryContextSwitchTo(TopMemoryContext);
 
-	old_context = MemoryContextSwitchTo(TopMemoryContext);
+	elog(NOTICE, "Goodbye from PostGIS Raster %s", POSTGIS_VERSION);
 
 	/* Clean up */
 	pfree(env_postgis_gdal_enabled_drivers);
