@@ -341,8 +341,14 @@ ORDER BY 2;
 
 -- Make sure all postgis functions are owned by the
 -- same role as postgis_lib_version
-SELECT DISTINCT 'unexpected ownership', proname || ':' || proowner::regrole
-FROM pg_proc
+SELECT DISTINCT
+	'unexpected ownership: ' || oid::regprocedure ||
+	' is owned by ' ||  proowner::regrole ||
+	' instead of ' || (
+			SELECT proowner::regrole FROM pg_proc
+			WHERE proname = 'postgis_lib_version'
+	)
+	FROM pg_proc
 WHERE (
 	probin like '%postgis%'
   OR (
@@ -357,7 +363,21 @@ AND proowner !=
 	proowner
 	FROM pg_proc WHERE proname = 'postgis_lib_version'
 )
-ORDER BY 2;
+ORDER BY 1;
+
+-- Make sure all postgis functions are SECURITY INVOKER
+SELECT DISTINCT 'unexpected security definer', oid::regprocedure, prosecdef
+FROM pg_proc
+WHERE (
+	probin like '%postgis%'
+  OR (
+		probin is null and
+		oid::regprocedure::text like E'st\\_%' or
+		oid::regprocedure::text like 'postgis_%'
+	)
+)
+AND prosecdef
+ORDER BY oid;
 
 SELECT 'UNEXPECTED', postgis_full_version()
 	WHERE postgis_full_version() LIKE '%UNPACKAGED%'
